@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <limits.h>
@@ -55,15 +55,9 @@ static const int chromaBits[4] = { 6, 6, 4+1<<CHROMA_422, 4+1<<CHROMA_444 };
 static const int mv_ffmpeg_to_xvmc[8];
 static const char* mv_ffmpeg_to_string[8];
 
-//#include "xvmc_debug.h"
-
-#undef fprintf
-
 static inline xvmc_render_state_t *render_state(const MpegEncContext *s);
 static inline int calc_cbp(MpegEncContext *s);
 static void set_block_pattern(const MpegEncContext *, XvMCMacroBlock *);
-static XvMCSurface* find_past_surface(MpegEncContext *, xvmc_render_state_t *);
-static XvMCSurface* find_future_surface(MpegEncContext *);
 static void setup_pmv(MpegEncContext *, XvMCMacroBlock *);
 static void setup_pmv_frame(MpegEncContext *, XvMCMacroBlock *);
 static void setup_pmv_field(MpegEncContext *, XvMCMacroBlock *);
@@ -76,6 +70,8 @@ static inline void handle_intra_block(const MpegEncContext *, const XvMCMacroBlo
 static inline void handle_p_b_block(const MpegEncContext *, const XvMCMacroBlock *,
                                     xvmc_render_state_t *render);
 
+#include "xvmccommon.c"
+
 //set s->block
 void XVMC_init_block(MpegEncContext *s)
 {
@@ -96,52 +92,9 @@ const int mb_block_count = 4+(1<<s->chroma_format);
            s->pblocks[i] = NULL;
         }
         cbp+=cbp;
-//        printf("s->pblocks[%d]=%p ,s->block=%p cbp=%d\n",i,s->pblocks[i],s->block,cbp);
     }
 }
 
-XvMCSurface* findPastSurface(MpegEncContext *s, xvmc_render_state_t *render) {
-    Picture *lastp = s->last_picture_ptr;
-    xvmc_render_state_t *last = NULL;
-    
-    if (NULL!=lastp) {
-        last = (xvmc_render_state_t*)(lastp->data[2]);
-        if (B_TYPE==last->pict_type)
-            fprintf(stderr, "Past frame is a B frame in findPastSurface, this is bad.\n");
-        //assert(B_TYPE!=last->pict_type);
-    }
-
-    if (NULL==last) 
-        if (!s->first_field)
-            last = render; // predict second field from the first
-        else
-            return 0;
-    
-    if (last->magic != MP_XVMC_RENDER_MAGIC) 
-        return 0;
-
-    return (last->state & MP_XVMC_STATE_PREDICTION) ? last->p_surface : 0;
-}
-
-XvMCSurface* findFutureSurface(MpegEncContext *s) {
-    Picture *nextp = s->next_picture_ptr;
-    xvmc_render_state_t *next = NULL;
-    
-    if (NULL!=nextp) {
-        next = (xvmc_render_state_t*)(nextp->data[2]);
-        if (B_TYPE==next->pict_type)
-            fprintf(stderr, "Next frame is a B frame in findFutureSurface, this is bad.\n");
-        //assert(B_TYPE!=next->pict_type);
-    }
-
-    assert(NULL!=next);
-
-    if (next->magic != MP_XVMC_RENDER_MAGIC)
-        return 0;
-
-    return (next->state & MP_XVMC_STATE_PREDICTION) ? next->p_surface : 0;
-}
-
 //these functions should be called on every new field or/and frame
 //They should be safe if they are called few times for same field!
 int XVMC_field_start(MpegEncContext*s, AVCodecContext *avctx){
@@ -178,15 +131,18 @@ xvmc_render_state_t * render,* last, * next;
             render->p_past_surface = findPastSurface(s, render);
             render->p_future_surface = findFutureSurface(s);
             if (!render->p_past_surface)
-                fprintf(stderr, "error: decoding B frame and past frame is null!");
+                av_log(s->avctx, AV_LOG_ERROR, "XvMC: Error, decoding "
+                       "B frame and past frame is null!\n");
             else if (!render->p_future_surface)
-                fprintf(stderr, "error: decoding B frame and future frame is null!");
+                av_log(s->avctx, AV_LOG_ERROR, "XvMC: Error, decoding "
+                       "B frame and future frame is null!\n");
             else return 0;
             return 0; // pretend things are ok even if they aren't
         case  P_TYPE:
             render->p_past_surface = findPastSurface(s, render);
             if (!render->p_past_surface)
-                fprintf(stderr, "error: decoding P frame and past frame is null!");
+                av_log(s->avctx, AV_LOG_ERROR, "XvMC: error, decoding "
+                       "P frame and past frame is null!\n");
             else return 0;
             return 0; // pretend things are ok even if they aren't
      }
@@ -295,12 +251,12 @@ static inline void setup_context(MpegEncContext *s)
     if (s->encoding)
     {
         av_log(s->avctx, AV_LOG_ERROR, "XVMC doesn't support encoding!!!\n");
-        av_abort();
+        return -1;
     }
     // Do I need to export quant when I could not perform postprocessing?
     // Anyway, it doesn't hurt.
     s->current_picture.qscale_table[s->mb_y * s->mb_stride + s->mb_x] = s->qscale;
-    s->mb_skiped = 0; //MC doesn't skip blocks
+    s->mb_skipped = 0; //MC doesn't skip blocks
 }
 
 static inline XvMCMacroBlock *macroblock(const xvmc_render_state_t *render,
@@ -327,7 +283,8 @@ static inline void DBG_printPictureStructureChanges(int picture_structure)
     int frame = (PICT_FRAME==picture_structure)? 1 : 0;
     if (oldframe != frame)
     {
-        fprintf(stderr, "picture structure %s\n", frame ? "FRAME" : "FIELD");
+        av_log(NULL, AV_LOG_ERROR, "XvMC: picture structure %s\n",
+               frame ? "FRAME" : "FIELD");
         oldframe = frame;
     }
 #endif
@@ -533,10 +490,8 @@ static void setup_pmv_frame(MpegEncContext *s,
             mv_block->PMV[1][1][1] = s->mv[1][1][1]<<1;
             mv_block->motion_vertical_field_select = mvfs0 | mvfs1;
         }
-        else
+        else if (MV_TYPE_DMV==s->mv_type)
         {
-            assert(MV_TYPE_DMV==s->mv_type);
-            //fprintf(stdout, "XVMC: DMV ");
             mv_block->PMV[0][0][0] = s->mv[0][0][0]; // top    from top
             mv_block->PMV[0][0][1] = s->mv[0][0][1]<<1;
             mv_block->PMV[0][1][0] = s->mv[0][0][0]; // bottom from bottom
@@ -546,6 +501,8 @@ static void setup_pmv_frame(MpegEncContext *s,
             mv_block->PMV[1][1][0] = s->mv[0][3][0]; // bottom from top
             mv_block->PMV[1][1][1] = s->mv[0][3][1]<<1;
         }
+        else
+            av_log(s->avctx, AV_LOG_ERROR, "XvMC: Unexpected movement type\n");
     }
     else
     {
@@ -557,7 +514,8 @@ static void setup_pmv_frame(MpegEncContext *s,
     // Error conditions
     if (MV_TYPE_16X8 == s->mv_type)
     {
-        fprintf(stderr, "XVMC: Field motion vector type with frame picture!\n");
+        av_log(s->avctx, AV_LOG_ERROR,
+               "XvMC: Field motion vector type with frame picture!\n");
         if (s->mv_type == MV_TYPE_16X8)
         {
             int mvfs0 = (s->field_select[0][0]) ? XVMC_SELECT_FIRST_FORWARD   : 0;
@@ -570,10 +528,11 @@ static void setup_pmv_frame(MpegEncContext *s,
     if (-1 == mv_block->motion_type)
     {
         if (MV_TYPE_8X8==s->mv_type)
-            fprintf(stderr, "XVMC: H263/MPEG4 motion"
-                    " vectortype in MPEG2 decoder!\n");
+            av_log(s->avctx, AV_LOG_ERROR,
+                   "XvMC: H263/MPEG4 motion vectortype in MPEG2 decoder!\n");
         else
-            fprintf(stderr, "XVMC: Unknown motion vector type!\n");
+            av_log(s->avctx, AV_LOG_ERROR,
+                   "XvMC: Unknown motion vector type!\n");
         mv_block->motion_type=XVMC_PREDICTION_FRAME;
     }
 }
@@ -613,9 +572,11 @@ static void setup_pmv_field(MpegEncContext *s,
     if (-1==mv_block->motion_type)
     {
         if (MV_TYPE_8X8==s->mv_type) 
-            fprintf(stderr, "XVMC: H263/MPEG4 motion vector type in MPEG2 decoder!\n");
+            av_log(s->avctx, AV_LOG_ERROR,
+                   "XvMC: H263/MPEG4 motion vector type in MPEG2 decoder!\n");
         else
-            fprintf(stderr, "XVMC: Unknown motion vector type!\n");
+            av_log(s->avctx, AV_LOG_ERROR,
+                   "XvMC: Unknown motion vector type!\n");
         mv_block->motion_type = XVMC_PREDICTION_FRAME;
     }
 }
