@@ -99,6 +99,8 @@ class MythContextPrivate;
       "upnp debugging messages")                 \
     F(VB_SOCKET,    0x00100000, "socket",    1,  \
       "socket debugging messages")               \
+    F(VB_MEDIA,     0x00800000, "media",     1,  \
+      "Media Manager debugging messages")        \
     F(VB_TIMESTAMP, 0x80000000, "timestamp", 1,  \
       "Conditional data driven messages")        \
     F(VB_NONE,      0x00000000, "none",      0,  \
@@ -222,14 +224,14 @@ class MythPrivRequest
 
 /// Update this whenever the plug-in API changes.
 /// Including changes in the libmythtv class methods used by plug-ins.
-#define MYTH_BINARY_VERSION "0.20.20060828-3"
+#define MYTH_BINARY_VERSION "0.20.20060828-4"
 
 /** \brief Increment this whenever the MythTV network protocol changes.
  *
  *   You must also update this value in
  *   mythplugins/mythweb/includes/mythbackend.php
  */
-#define MYTH_PROTO_VERSION "30"
+#define MYTH_PROTO_VERSION "31"
 
 /** \class MythContext
  *  \brief This class contains the runtime context for MythTV.
