@@ -1,7 +1,6 @@
 #ifndef UTIL_H_
 #define UTIL_H_
 
-#include <qsocket.h>
 #include <qstringlist.h>
 #include <qdatetime.h>
 #include <qcolor.h>
@@ -39,23 +38,9 @@ class MythTimer
 
 QDateTime mythCurrentDateTime();
 
-QString SocDevErrStr(int error);
-
-// QSockectDevice (frontend)
-bool connectSocket(QSocketDevice *socket, const QString &host, uint port);
-
-bool WriteStringList(QSocketDevice *socket, QStringList &list);
-bool ReadStringList(QSocketDevice *socket, QStringList &list,
-                    bool quickTimeout = false);
-
-bool WriteBlock(QSocketDevice *socket, void *data, uint len);
-
-// QSockect (backend)
-bool WriteStringList(QSocket *socket, QStringList &list);
-bool ReadStringList(QSocket *socket, QStringList &list);
-
-bool WriteBlock(QSocket *socket, void *data, uint len);
-int ReadBlock(QSocket *socket, void *data, uint maxlen);
+// This is necessary for GCC 3.3, which has llabs(long long)
+// but not abs(long long) or std::llabs(long long)
+inline  long long  absLongLong(long long  n)  { return n >= 0 ? n : -n; } 
 
 void encodeLongLong(QStringList &list, long long num);
 long long decodeLongLong(QStringList &list, uint offset);
@@ -77,4 +62,5 @@ long long getDiskSpace(const QString&,long long&,long long&);
 bool getUptime(time_t &uptime);
 bool getMemStats(int &totalMB, int &freeMB, int &totalVM, int &freeVM);
 
+void myth_eject(void);
 #endif // UTIL_H_
