@@ -1,20 +1,41 @@
-#include <iostream>
+// -*- Mode: c++ -*-
+
+// C headers
+#include <ctime>
+
+// POSIX headers
 #include <pthread.h>
-#include <cstdio>
-#include <cstdlib>
-#include <cerrno>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <inttypes.h>
+
+// System headers
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
-#include <ctime>
-#include <qregexp.h>
-#include "videodev_myth.h"
 
+#include <algorithm>
+using namespace std;
+
+// Qt headers
+#include <qregexp.h>
+
+// avlib headers
+#include "../libavcodec/avcodec.h"
+
+// MythTV headers
+#include "mpegrecorder.h"
+#include "RingBuffer.h"
+#include "mythcontext.h"
+#include "programinfo.h"
+#include "recordingprofile.h"
+#include "tv_rec.h"
+#include "videodev_myth.h"
+#include "util.h"
+
+// ivtv header
 extern "C" {
-#include <inttypes.h>
 #ifdef USING_IVTV_HEADER
 #include <linux/ivtv.h>
 #else
@@ -22,65 +43,66 @@ extern "C" {
 #endif
 }
 
-using namespace std;
+#define LOC QString("MPEGRec(%1): ").arg(videodevice)
+#define LOC_WARN QString("MPEGRec(%1) Warning: ").arg(videodevice)
+#define LOC_ERR QString("MPEGRec(%1) Error: ").arg(videodevice)
 
-#include "mpegrecorder.h"
-#include "RingBuffer.h"
-#include "mythcontext.h"
-#include "programinfo.h"
-#include "recordingprofile.h"
-#include "tv_rec.h"
+const int MpegRecorder::audRateL1[] =
+{
+    32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, 0
+};
 
-extern "C" {
-#include "../libavcodec/avcodec.h"
-}
+const int MpegRecorder::audRateL2[] =
+{
+    32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384, 0
+};
 
-const int MpegRecorder::audRateL1[] = { 32, 64, 96, 128, 160, 192, 224, 
-                                        256, 288, 320, 352, 384, 416, 448, 0 };
-const int MpegRecorder::audRateL2[] = { 32, 48, 56, 64, 80, 96, 112, 128, 
-                                        160, 192, 224, 256, 320, 384, 0 };
-const char* MpegRecorder::streamType[] = { "MPEG-2 PS", "MPEG-2 TS", 
-                                           "MPEG-1 VCD", "PES AV", "", "PES V", 
-                                           "", "PES A", "", "", "DVD", "VCD",
-                                           "SVCD", "DVD-Special 1", 
-                                           "DVD-Special 2", 0 };
-const char* MpegRecorder::aspectRatio[] = { "Square", "4:3", "16:9", 
-                                            "2.21:1", 0 };
+const char* MpegRecorder::streamType[] =
+{
+    "MPEG-2 PS", "MPEG-2 TS",     "MPEG-1 VCD",    "PES AV",
+    "",          "PES V",          "",             "PES A",
+    "",          "",              "DVD",           "VCD",
+    "SVCD",      "DVD-Special 1", "DVD-Special 2", 0
+};
 
-#define BUILDBUFFERMAX (1024 * 1024)
+const char* MpegRecorder::aspectRatio[] =
+{
+    "Square", "4:3", "16:9", "2.21:1", 0
+};
+
+const unsigned int MpegRecorder::kBuildBufferMaxSize = 1024 * 1024;
 
 MpegRecorder::MpegRecorder(TVRec *rec) :
-    RecorderBase(rec, "MpegRecorder")
+    RecorderBase(rec),
+    // Debugging variables
+    deviceIsMpegFile(false),
+    bufferSize(4096),
+    // State
+    recording(false),         encoding(false),
+    errored(false),
+    // Pausing state
+    cleartimeonpause(false),
+    // Number of frames written
+    framesWritten(0),
+    // Encoding info
+    width(720),               height(480),
+    bitrate(4500),            maxbitrate(6000),
+    streamtype(0),            aspectratio(2),
+    audtype(2),               audsamplerate(48000),
+    audbitratel1(14),         audbitratel2(14),
+    audvolume(80),
+    // Input file descriptors
+    chanfd(-1),               readfd(-1),
+    // Keyframe tracking inforamtion
+    keyframedist(15),         gopset(false),
+    leftovers(0),             lastpackheaderpos(0),
+    lastseqstart(0),          numgops(0),
+    // Position map support
+    positionMapLock(false),
+    // buffer used for ...
+    buildbuffer(new unsigned char[kBuildBufferMaxSize + 1]),
+    buildbuffersize(0)
 {
-    errored = false;
-    recording = false;
-
-    framesWritten = 0;
-
-    chanfd = -1; 
-    readfd = -1;
-
-    width = 720;
-    height = 480;
-
-    keyframedist = 15;
-    gopset = false;
-
-    bitrate = 4500;
-    maxbitrate = 6000;
-    streamtype = 0;
-    aspectratio = 2;
-    audtype = 2;
-    audsamplerate = 48000;
-    audbitratel1 = 14;
-    audbitratel2 = 14;
-    audvolume = 80;
-
-    deviceIsMpegFile = false;
-    bufferSize = 4096;
-
-    buildbuffer = new unsigned char[BUILDBUFFERMAX + 1];
-    buildbuffersize = 0;
 }
 
 MpegRecorder::~MpegRecorder()
@@ -89,12 +111,6 @@ MpegRecorder::~MpegRecorder()
     delete [] buildbuffer;
 }
 
-void MpegRecorder::deleteLater(void)
-{
-    TeardownAll();
-    RecorderBase::deleteLater();
-}
-
 void MpegRecorder::TeardownAll(void)
 {
     if (chanfd >= 0)
@@ -133,9 +149,12 @@ void MpegRecorder::SetOption(const QString &opt, int value)
                 break;
             }
         }
-        if (!found)
-            cerr << "Audiorate(L1): " << value << " is invalid\n";
 
+        if (!found)
+        {
+            VERBOSE(VB_IMPORTANT, LOC_ERR + "Audiorate(L1): " +
+                    QString("%1 is invalid").arg(value));
+        }
     }
     else if (opt == "mpeg2audbitratel2")
     {
@@ -148,8 +167,12 @@ void MpegRecorder::SetOption(const QString &opt, int value)
                 break;
             }
         }
+
         if (!found)
-            cerr << "Audiorate(L2): " << value << " is invalid\n";
+        {
+            VERBOSE(VB_IMPORTANT, LOC_ERR + "Audiorate(L2): " +
+                    QString("%1 is invalid").arg(value));
+        }
     }
     else if (opt == "mpeg2audvolume")
         audvolume = value;
@@ -173,7 +196,10 @@ void MpegRecorder::SetOption(const QString &opt, const QString &value)
         }
 
         if (!found)
-            cerr << "MPEG2 stream type: " << value << " is invalid\n";
+        {
+            VERBOSE(VB_IMPORTANT, LOC_ERR + "MPEG2 stream type: " +
+                    QString("%1 is invalid").arg(value));
+        }
     }
     else if (opt == "mpeg2aspectratio")
     {
@@ -189,7 +215,10 @@ void MpegRecorder::SetOption(const QString &opt, const QString &value)
         }
 
         if (!found)
-            cerr << "MPEG2 Aspect-ratio: " << value << " is invalid\n";
+        {
+            VERBOSE(VB_IMPORTANT, LOC_ERR + "MPEG2 Aspect-ratio: " +
+                    QString("%1 is invalid").arg(value));
+        }
     }
     else if (opt == "mpeg2audtype")
     {
@@ -198,11 +227,15 @@ void MpegRecorder::SetOption(const QString &opt, const QString &value)
         else if (value == "Layer II")
             audtype = 2;
         else
-            cerr << "MPEG2 layer: " << value << " is invalid\n";
+        {
+            VERBOSE(VB_IMPORTANT, LOC_ERR + "MPEG2 audio layer: " +
+                    QString("%1 is invalid").arg(value));
+        }
     }
     else
+    {
         RecorderBase::SetOption(opt, value);
-
+    }
 }
 
 void MpegRecorder::SetOptionsFromProfile(RecordingProfile *profile, 
@@ -252,8 +285,9 @@ bool MpegRecorder::OpenMpegFileAsInput(void)
 
     if (readfd < 0)
     {
-        cerr << "Can't open MPEG File: " << videodevice << endl;
-        perror("open mpeg file:");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + QString("Can't open MPEG File '%1'")
+                .arg(videodevice) + ENO);
+
         return false;
     }
     return true;
@@ -264,20 +298,18 @@ bool MpegRecorder::OpenV4L2DeviceAsInput(void)
     chanfd = open(videodevice.ascii(), O_RDWR);
     if (chanfd < 0)
     {
-        cerr << "Can't open video device: " << videodevice << endl;
-        perror("open video:");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "Can't open video device. " + ENO);
         return false;
     }
 
     struct v4l2_format vfmt;
-    memset(&vfmt, 0, sizeof(vfmt));
+    bzero(&vfmt, sizeof(vfmt));
 
     vfmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 
     if (ioctl(chanfd, VIDIOC_G_FMT, &vfmt) < 0)
     {
-        cerr << "Error getting format\n";
-        perror("VIDIOC_G_FMT:");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "Error getting format" + ENO);
         return false;
     }
 
@@ -286,129 +318,229 @@ bool MpegRecorder::OpenV4L2DeviceAsInput(void)
 
     if (ioctl(chanfd, VIDIOC_S_FMT, &vfmt) < 0)
     {
-        cerr << "Error setting format\n";
-        perror("VIDIOC_S_FMT:");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "Error setting format" + ENO);
         return false;
     }
 
-    struct ivtv_ioctl_codec ivtvcodec;
-    memset(&ivtvcodec, 0, sizeof(ivtvcodec));
-
-    if (ioctl(chanfd, IVTV_IOC_G_CODEC, &ivtvcodec) < 0)
-    {
-        cerr << "Error getting codec params\n";
-        perror("IVTV_IOC_G_CODEC:");
-        return false;
-    }
-
-    // only 48kHz works properly.
-    int audio_rate = 1;
-
-/*
-    int audio_rate;
-    if (audsamplerate == 44100)
-        audio_rate = 0;
-    else if (audsamplerate == 48000)
-        audio_rate = 1;
-    // 32kHz doesn't seem to work, let's force 44.1kHz instead.
-    else if (audsamplerate == 32000)
-        audio_rate = 2;
-    else
-    {
-        cerr << "Error setting audio sample rate\n";
-        cerr << audsamplerate << " is not a valid sampling rate\n";
-        return;
-    }
-*/
-
-    ivtvcodec.aspect = aspectratio;
-    if (audtype == 2)
-    {
-        if (audbitratel2 < 10)
-            audbitratel2 = 10;
-        ivtvcodec.audio_bitmask = audio_rate + (audtype << 2) + 
-                                  (audbitratel2 << 4);
-    }
-    else
-    {
-        if (audbitratel1 < 6)
-            audbitratel1 = 6;
-        ivtvcodec.audio_bitmask = audio_rate + (audtype << 2) + 
-                                  (audbitratel1 << 4);
-    }
-
-    ivtvcodec.bitrate = bitrate * 1000;
-    ivtvcodec.bitrate_peak = maxbitrate * 1000;
-
-    if (ivtvcodec.bitrate > ivtvcodec.bitrate_peak)
-        ivtvcodec.bitrate = ivtvcodec.bitrate_peak;
-
-    // framerate (1 = 25fps, 0 = 30fps)
-    ivtvcodec.framerate = (!ntsc_framerate);
-    ivtvcodec.stream_type = streamtype;
-
-    if (ioctl(chanfd, IVTV_IOC_S_CODEC, &ivtvcodec) < 0)
-    {
-        cerr << "Error setting codec params\n";
-        perror("IVTV_IOC_S_CODEC:");
-        return false;
-    }
-
-    if (ivtvcodec.framerate)
-        keyframedist = 12;
-
     struct v4l2_control ctrl;
     ctrl.id = V4L2_CID_AUDIO_VOLUME;
     ctrl.value = 65536 / 100 *audvolume;
 
     if (ioctl(chanfd, VIDIOC_S_CTRL, &ctrl) < 0)
     {
-        cerr << "Error setting codec params\n";
-        perror("VIDIOC_S_CTRL:");
+        VERBOSE(VB_IMPORTANT, LOC_WARN +
+                "Unable to set recording volume" + ENO + "\n\t\t\t" +
+                "If you are using an AverMedia M179 card this is normal.");
+    }
+
+    if (!SetIVTVDeviceOptions(chanfd) && !SetV4L2DeviceOptions(chanfd))
+        return false;
+
+    readfd = open(videodevice.ascii(), O_RDWR | O_NONBLOCK);
+    if (readfd < 0)
+    {
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "Can't open video device." + ENO);
         return false;
     }
 
-    if (vbimode) {
+    return true;
+}
+
+bool MpegRecorder::SetIVTVDeviceOptions(int chanfd)
+{
+    struct ivtv_ioctl_codec ivtvcodec;
+    bzero(&ivtvcodec, sizeof(ivtvcodec));
+
+    if (ioctl(chanfd, IVTV_IOC_G_CODEC, &ivtvcodec) < 0)
+    {
+        // Downgrade to VB_RECORD warning when ioctl isn't supported,
+        // unless the driver is completely missitng it probably
+        // supports the Linux v2.6.18 v4l2 mpeg encoder API.
+        VERBOSE((22 == errno) ? VB_RECORD : VB_IMPORTANT,
+                ((22 == errno) ? LOC_WARN : LOC_ERR) +
+                "Error getting codec params using old IVTV ioctl" + ENO);
+        return false;
+    }
+
+    audbitratel1 = max(audbitratel1, 6);
+    audbitratel2 = max(audbitratel2, 10);
+
+    int audio_rate = 1; // only 48kHz works properly.
+    int audbitrate = (2 == audtype) ? audbitratel2 : audbitratel1;
+
+    ivtvcodec.audio_bitmask  = audio_rate | (audtype << 2);
+    ivtvcodec.audio_bitmask |= audbitrate << 4;
+    ivtvcodec.aspect         = aspectratio;
+    ivtvcodec.bitrate        = min(bitrate, maxbitrate) * 1000;
+    ivtvcodec.bitrate_peak   = maxbitrate * 1000;
+    ivtvcodec.framerate      = ntsc_framerate ? 0 : 1; // 1->25fps, 0->30fps
+    ivtvcodec.stream_type    = streamtype;
+
+    if (ioctl(chanfd, IVTV_IOC_S_CODEC, &ivtvcodec) < 0)
+    {
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "Error setting codec params" + ENO);
+        return false;
+    }
+
+    keyframedist = (ivtvcodec.framerate) ? 12 : keyframedist;
+
+    if (vbimode)
+    {
         struct ivtv_sliced_vbi_format vbifmt;
-        memset(&vbifmt, 0, sizeof(struct ivtv_sliced_vbi_format));
-        vbifmt.service_set = (1==vbimode) ? VBI_TYPE_TELETEXT : VBI_TYPE_CC;
+        bzero(&vbifmt, sizeof(struct ivtv_sliced_vbi_format));
+        vbifmt.service_set = (1 == vbimode) ? VBI_TYPE_TELETEXT : VBI_TYPE_CC;
+
         if (ioctl(chanfd, IVTV_IOC_S_VBI_MODE, &vbifmt) < 0) 
         {
             struct v4l2_format vbi_fmt;
-            memset(&vbi_fmt, 0, sizeof(struct v4l2_format));
+            bzero(&vbi_fmt, sizeof(struct v4l2_format));
             vbi_fmt.type = V4L2_BUF_TYPE_SLICED_VBI_CAPTURE;
-            vbi_fmt.fmt.sliced.service_set = 0;
-            if (1 == vbimode)
-                vbi_fmt.fmt.sliced.service_set |= V4L2_SLICED_VBI_625;
-            else
-                vbi_fmt.fmt.sliced.service_set |= V4L2_SLICED_VBI_525;
+            vbi_fmt.fmt.sliced.service_set = (1 == vbimode) ?
+                V4L2_SLICED_VBI_625 : V4L2_SLICED_VBI_525;
 
             if (ioctl(chanfd, VIDIOC_S_FMT, &vbi_fmt) < 0)
             {
-                VERBOSE(VB_IMPORTANT, QString("Can't enable VBI recording"));
-                perror("vbi");
+                VERBOSE(VB_IMPORTANT, "Can't enable VBI recording" + ENO);
             }
         }
 
         int embedon = 1;
         if (ioctl(chanfd, IVTV_IOC_S_VBI_EMBED, &embedon) < 0)
         {
-            VERBOSE(VB_IMPORTANT, QString("Can't enable VBI recording (2)"));
-            perror("vbi");
+            VERBOSE(VB_IMPORTANT, LOC + "Can't enable VBI recording (2)"+ENO);
         }
 
         ioctl(chanfd, IVTV_IOC_G_VBI_MODE, &vbifmt);
-        VERBOSE(VB_RECORD, QString("VBI service:%1, packet size:%2, io size:%3").
-                arg(vbifmt.service_set).arg(vbifmt.packet_size).arg(vbifmt.io_size));
+
+        VERBOSE(VB_RECORD, LOC + QString(
+                    "VBI service:%1, packet size:%2, io size:%3")
+                .arg(vbifmt.service_set).arg(vbifmt.packet_size)
+                .arg(vbifmt.io_size));
     }
 
-    readfd = open(videodevice.ascii(), O_RDWR | O_NONBLOCK);
-    if (readfd < 0)
+    return true;
+}
+
+bool MpegRecorder::SetV4L2DeviceOptions(int chanfd)
+{
+    static const int kNumControls = 7;
+    struct v4l2_ext_controls ctrls;
+    struct v4l2_ext_control ext_ctrl[kNumControls];
+
+    // Set controls
+    bzero(&ctrls,    sizeof(struct v4l2_ext_controls));
+    bzero(&ext_ctrl, sizeof(struct v4l2_ext_control) * kNumControls);
+
+    audbitratel1 = max(audbitratel1, 6);
+    audbitratel2 = max(audbitratel2, 10);
+
+    int audbitrate = (2 == audtype) ? audbitratel2 : audbitratel1;
+
+    if (audtype != 2)
     {
-        cerr << "Can't open video device: " << videodevice << endl;
-        perror("open video:");
+        VERBOSE(VB_IMPORTANT, LOC_WARN +
+                "MPEG Layer1 does not work properly with ivtv driver. "
+                "\n\t\t\tUsing MPEG layer 2 audio instead.");
+    }
+
+    // only 48kHz works properly.
+    ext_ctrl[0].id    = V4L2_CID_MPEG_AUDIO_SAMPLING_FREQ;
+    ext_ctrl[0].value = V4L2_MPEG_AUDIO_SAMPLING_FREQ_48000;
+
+    ext_ctrl[1].id    = V4L2_CID_MPEG_VIDEO_ASPECT;
+    ext_ctrl[1].value = aspectratio - 1;
+
+    ext_ctrl[2].id    = V4L2_CID_MPEG_AUDIO_ENCODING;
+    ext_ctrl[2].value = audtype - 1;
+
+    ext_ctrl[3].id    = V4L2_CID_MPEG_AUDIO_L2_BITRATE;
+    ext_ctrl[3].value = audbitrate - 1;
+
+    ext_ctrl[4].id    = V4L2_CID_MPEG_VIDEO_BITRATE;
+    ext_ctrl[4].value = (bitrate = min(bitrate, maxbitrate)) * 1000;
+
+    ext_ctrl[5].id    = V4L2_CID_MPEG_VIDEO_BITRATE_PEAK;
+    ext_ctrl[5].value = maxbitrate * 1000;
+
+    ext_ctrl[6].id    = V4L2_CID_MPEG_STREAM_TYPE;
+    ext_ctrl[6].value = V4L2_MPEG_STREAM_TYPE_MPEG2_PS;
+
+    ctrls.ctrl_class  = V4L2_CTRL_CLASS_MPEG;
+    ctrls.count       = kNumControls;
+    ctrls.controls    = ext_ctrl;
+
+    if (ioctl(chanfd, VIDIOC_S_EXT_CTRLS, &ctrls) < 0)
+    {
+        if (ctrls.error_idx >= ctrls.count)
+        {
+            VERBOSE(VB_IMPORTANT, LOC_ERR +
+                    "Could not set MPEG controls" + ENO);
+        }
+        else
+        {
+            VERBOSE(VB_IMPORTANT, LOC_ERR +
+                    QString("Could not set MPEG controls %1 through %2.")
+                    .arg(ctrls.error_idx)
+                    .arg(ext_ctrl[ctrls.error_idx].value) + ENO);
+        }
         return false;
     }
+
+    // Get controls
+    ext_ctrl[0].id    = V4L2_CID_MPEG_VIDEO_GOP_SIZE;
+    ext_ctrl[0].value = 0;
+
+    ctrls.ctrl_class  = V4L2_CTRL_CLASS_MPEG;
+    ctrls.count       = 1;
+    ctrls.controls    = ext_ctrl;
+
+    if (ioctl(chanfd, VIDIOC_G_EXT_CTRLS, &ctrls) < 0)
+    {
+        VERBOSE(VB_IMPORTANT, LOC_WARN + "Unable to get "
+                "V4L2_CID_MPEG_VIDEO_GOP_SIZE, defaulting to 12" + ENO);
+        ext_ctrl[0].value = 12;
+    }
+
+    keyframedist = ext_ctrl[0].value;
+
+    // VBI is not yet working as of July 11th, 2006 with IVTV driver.
+    // V4L2_CID_MPEG_STREAM_VBI_FMT needs to be finished/supported first.
+    if (vbimode)
+    {
+        struct v4l2_format vbifmt;
+        bzero(&vbifmt, sizeof(struct v4l2_format));
+        vbifmt.type = V4L2_BUF_TYPE_SLICED_VBI_CAPTURE;
+        /* vbifmt.fmt.sliced.service_set = (1==vbimode) ?
+                                           VBI_TYPE_TELETEXT : VBI_TYPE_CC; */
+        if (1 == vbimode)
+            vbifmt.fmt.sliced.service_set |= V4L2_SLICED_VBI_625;
+        else
+            vbifmt.fmt.sliced.service_set |= V4L2_SLICED_VBI_525;
+
+        if (ioctl(chanfd, VIDIOC_S_FMT, &vbifmt) < 0)
+        {
+            VERBOSE(VB_IMPORTANT, "Can't enable VBI recording" + ENO);
+        }
+
+        struct v4l2_ext_control vbi_fmt;
+        vbi_fmt.id    = V4L2_CID_MPEG_STREAM_VBI_FMT;
+        vbi_fmt.value = V4L2_MPEG_STREAM_VBI_FMT_IVTV;
+        ctrls.ctrl_class = V4L2_CTRL_CLASS_MPEG;
+        ctrls.count      = 1;
+        ctrls.controls   = &vbi_fmt;
+        if (ioctl(chanfd, VIDIOC_S_EXT_CTRLS, &ctrls) < 0)
+        {
+            VERBOSE(VB_IMPORTANT, LOC + "Can't enable VBI recording (2)"+ENO);
+        }
+
+        ioctl(chanfd, VIDIOC_G_FMT, &vbifmt);
+
+        VERBOSE(VB_RECORD, LOC + QString(
+                "VBI service:%1, io size:%3")
+                .arg(vbifmt.fmt.sliced.service_set)
+                .arg(vbifmt.fmt.sliced.io_size));
+    }
+
     return true;
 }
 
@@ -430,7 +562,7 @@ void MpegRecorder::StartRecording(void)
 
     if (!SetupRecording())
     {
-        VERBOSE(VB_IMPORTANT, "Error initializing recording");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "Failed to setup recording.");
 	errored = true;
 	return;
     }
@@ -477,11 +609,22 @@ void MpegRecorder::StartRecording(void)
             case -1:
                 if (errno == EINTR)
                     continue;
-                perror("select");
+
+                VERBOSE(VB_IMPORTANT, LOC_ERR + "Select error" + ENO);
                 continue;
+
             case 0:
-                printf("select timeout - ivtv driver has stopped responding\n");
+                VERBOSE(VB_IMPORTANT, LOC_ERR + "select timeout - "
+                        "ivtv driver has stopped responding");
+
+                if (close(readfd) != 0)
+                {
+                    VERBOSE(VB_IMPORTANT, LOC_ERR + "Close error" + ENO);
+                }
+
+                readfd = -1; // Force PVR card to be reopened on next iteration
                 continue;
+
            default: break;
         }
 
@@ -503,12 +646,15 @@ void MpegRecorder::StartRecording(void)
         }
         else if (ret < 0 && errno != EAGAIN)
         {
-            cerr << "error reading from: " << videodevice << endl;
-            perror("read");
+            VERBOSE(VB_IMPORTANT, LOC_ERR + QString("error reading from: %1")
+                    .arg(videodevice) + ENO);
+
             continue;
         }
         else if (ret > 0)
+        {
             ProcessData(buffer, ret);
+        }
     }
 
     FinishRecording();
@@ -517,11 +663,6 @@ void MpegRecorder::StartRecording(void)
     recording = false;
 }
 
-int MpegRecorder::GetVideoFd(void)
-{
-    return chanfd;
-}
-
 bool MpegRecorder::SetupRecording(void)
 {
     leftovers = 0xFFFFFFFF;
@@ -545,15 +686,6 @@ void MpegRecorder::FinishRecording(void)
     positionMapLock.unlock();
 }
 
-void MpegRecorder::SetVideoFilters(QString &filters)
-{
-    (void)filters;
-}
-
-void MpegRecorder::Initialize(void)
-{
-}
-
 #define PACK_HEADER   0x000001BA
 #define GOP_START     0x000001B8
 #define SEQ_START     0x000001B3
@@ -626,7 +758,7 @@ void MpegRecorder::ProcessData(unsigned char *buffer, int len)
 
     leftovers = state;
 
-    if (buildbuffersize + leftlen > BUILDBUFFERMAX)
+    if (buildbuffersize + leftlen > kBuildBufferMaxSize)
     {
         ringBuffer->Write(buildbuffer, buildbuffersize);
         buildbuffersize = 0;
@@ -671,16 +803,6 @@ void MpegRecorder::Pause(bool clear)
     request_pause = true;
 }
 
-bool MpegRecorder::IsRecording(void)
-{
-    return recording;
-}
-
-long long MpegRecorder::GetFramesWritten(void)
-{
-    return framesWritten;
-}
-
 long long MpegRecorder::GetKeyframePosition(long long desired)
 {
     QMutexLocker locker(&positionMapLock);
@@ -709,7 +831,7 @@ void MpegRecorder::SetNextRecording(const ProgramInfo *progInf, RingBuffer *rb)
     }
 }
 
-/** \fn MpegRecorder::HandleKeyframe()
+/** \fn MpegRecorder::HandleKeyframe(void)
  *  \brief This save the current frame to the position maps
  *         and handles ringbuffer switching.
  */
@@ -745,7 +867,7 @@ void MpegRecorder::SavePositionMap(bool force)
     QMutexLocker locker(&positionMapLock);
 
     // save on every 5th key frame if in the first few frames of a recording
-    force |= (positionMap.size() < 30) && (positionMap.size()%5 == 1);
+    force |= (positionMap.size() < 30) && (positionMap.size() % 5 == 1);
     // save every 30th key frame later on
     force |= positionMapDelta.size() >= 30;
 
@@ -753,7 +875,9 @@ void MpegRecorder::SavePositionMap(bool force)
     {
         curRecording->SetPositionMapDelta(positionMapDelta,
                                           MARK_GOP_START);
-        curRecording->SetFilesize(lastpackheaderpos);
+        // Stop setting the filesize here until we get the contention issue
+        // between with this thread and the scheduler worked out.
+        //curRecording->SetFilesize(lastpackheaderpos);
         positionMapDelta.clear();
     }
 }
