@@ -11,10 +11,15 @@ class VideoOutputNull : public VideoOutput
 
     bool Init(int width, int height, float aspect, WId winid,
               int winx, int winy, int winw, int winh, WId embedid = 0);
+
+    bool SetupDeinterlace(bool, const QString &ovrf = "")
+        { (void)ovrf; return false; } // we don't deinterlace in null output..
+
     void PrepareFrame(VideoFrame *buffer, FrameScanType);
     void Show(FrameScanType );
 
-    void InputChanged(int width, int height, float aspect);
+    void InputChanged(int width, int height, float aspect,
+                      MythCodecID av_codec_id);
     void Zoom(int direction);
 
     void EmbedInWidget(WId wid, int x, int y, int w, int h);
