@@ -149,11 +149,13 @@ ManualBox::ManualBox(MythMainWindow *parent, const char *name)
     m_timer->start(300);
 
     gContext->addListener(this);
+    gContext->addCurrentLocation("ManualBox");
 }
 
 ManualBox::~ManualBox(void)
 {
     gContext->removeListener(this);
+    gContext->removeCurrentLocation();
     killPlayer();
 }
 
@@ -204,7 +206,16 @@ void ManualBox::startPlayer(void)
         tvstarting = true;
 
         m_tv = new TV();
-        m_tv->Init(false);
+        if (!m_tv->Init(false))
+        {
+            VERBOSE(VB_IMPORTANT, "Experienced fatal error initialzing "
+                    "TV class in ManualBox::startPlayer().");
+            delete m_tv;
+            m_tv = 0;
+            tvstarting = false;
+            return;
+        }
+        
 
         m_tv->EmbedOutput(m_pixlabel->winId(), 0, 0, (int)(320 * wmult),
                                                      (int)(240 * hmult));
@@ -276,6 +287,7 @@ void ManualBox::refreshTimeout(void)
 
     if (NULL != m_tv)
     {
+#if 0
         QString dummy;
         QString title;
         QString subtitle;
@@ -283,7 +295,8 @@ void ManualBox::refreshTimeout(void)
         m_tv->GetChannelInfo(NULL, title, subtitle, m_descString,
                              m_categoryString, m_startString, dummy,
                              dummy, dummy, dummy,
-                             m_chanidString, dummy, dummy);
+                             m_chanidString, dummy, dummy, dummy,
+                             dummy, dummy, dummy);
 
         if (m_lastStarttime != m_startString || m_lastChanid != m_chanidString)
         {
@@ -294,6 +307,7 @@ void ManualBox::refreshTimeout(void)
             m_lastStarttime = m_startString;
             m_lastChanid = m_chanidString;
         }
+#endif
     }
 }
 
@@ -313,7 +327,6 @@ void ManualBox::stopClicked(void)
 void ManualBox::startClicked(void)
 {
     ProgramInfo progInfo;
-    QSqlDatabase *db = QSqlDatabase::database();
 
     m_startButton->setEnabled(false);
 
@@ -329,8 +342,8 @@ void ManualBox::startClicked(void)
     progInfo.startts = now;
     progInfo.endts = now.addSecs(m_duration->value() * 60);
  
-    progInfo.Save(db);
+    progInfo.Save();
 
-    progInfo.ApplyRecordStateChange(db, kSingleRecord);
+    progInfo.ApplyRecordStateChange(kSingleRecord);
 }
 
